<?php
/*
Plugin Name: Talkomatic
Plugin URI: http://just.thinkofit.com/talkomatic-wordpress-plugin/
Description: Embed a Talkomatic chat room in a WordPress page or post
Version: 1.0
Author: David R. Woolley
Author URI: http://thinkofit.com/drwool
License: GPLv2
*/

/*  Copyright 2015 David R. Woolley  (email : drwool@thinkofit.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* 
	This plugin provides a shortcode for embedding a Talkomatic chat room in
	a WordPress page or post, using an iframe. 
	
	If the shortcode specifies a room name, any user who navigates to the page where
	this shortcode is included will automatically enter the named chat room. If no 
	room name is specified, the user will enter the Talkomatic lobby, which displays
	a list of active rooms and participants and allows the user to choose an existing
	room or open a new room, which may be public or private.
	
	If the named room is not already open, it will be opened automatically as a public room.
	If the named room is already open and is public, the user will enter the room if there is
	an open slot. (There is a maximum of five participants per room.) If the room is open as
	a private room, the user will be required to type the room key before being allowed in.
	The room key is defined by whoever opens the room and designates it as private.
	
	Talkomatic chat rooms do not place any burden on your WordPress server because they
	are hosted on the independent server at http://talko.cc. For more information about
	Talkomatic, see http://talko.cc/about.html
	
	
	Using the Talkomatic shortcode - Examples:
	
	Put user into a specific named room, with default iframe dimensions (900 x 850 pixels):
		[talkomatic room="Elbow Room"]
		
	Put user into a specific named room, with custom iframe dimensions:
		[talkomatic room="Broom Closet" width=600 height=800]
		
	Put user into the Talkomatic lobby, where they can choose a room or open a new one:
		[talkomatic]
		
	Put user into the Talkomatic lobby, with custom iframe dimensions:
		[talkomatic width=650 height=900]
	
	
	Sample HTML output from this shortcode:
	<iframe src="http://talko.cc/go?r=..zvvy..Broom.20.Closet" width="900" height="850"></iframe>
*/

function drw_talkomatic_func ( $attr ) {

	$width = isset($attr['width']) ? $attr['width'] : 900;
	$height = isset($attr['height']) ? $attr['height'] : 850;

	$out = '<iframe src="http://talko.cc';

	if (isset($attr['room'])) {
		$out .= '/go?r=' . toZvvyCode($attr['room'], '.');
	}

	$out .= '" width="' . $width . '" height="' . $height . '"></iframe>';
	return $out;
}


//
//	toZvvyCode: Convert a string to ZvvyCode encoding.
//		written by David R. Woolley, Thinkofit, Inc.	www.thinkofit.com
//
//	Arguments:
//		str		String to be encoded
//		escChr	Character code to use as the escape character, given as a numeric value.
//				(E.g., 92 or 0x5c to use backslash as the escape.)
//				MUST NOT BE A LETTER (a-z, A-Z) OR DIGIT (0-9). Any other character code is valid.
//	Returns:
//		ZvvyCode encoded string, or empty string if original string is empty
//

function toZvvyCode ($str, $escChr) {

	if ($str == '') {
		// If string is empty, return empty.
		return ('');
	}

	$lth = strlen($str);
	$escChrStr = chr($escChr);

	// Begin with two escape characters, the "zvvy" identifier, and two more escapes.
	$rs = $escChrStr.$escChrStr."zvvy".$escChrStr.$escChrStr;

	for ($i = 0; $i < $lth; $i++) {
		$ch = ord($str{$i});
		if (($ch >= 0x30 && $ch <= 0x39) || ($ch >= 0x41 && $ch <= 0x5A) || ($ch >= 0x61 && $ch <= 0x7A)) {
			// Copy a-z, A-Z, and 0-9 as-is.
			$rs .= $str{$i};
		}
		else {
			// Output a hex character value surrounded by escape characters.
			$rs .= $escChrStr . dechex($ch) . $escChrStr;
		}
	}
	return ($rs);
}


add_shortcode ('talkomatic', 'drw_talkomatic_func');

?>